// Oleg Petruny proprietary.


#include "RaycastInteractableActivator.h"

#include "DrawDebugHelpers.h" 
#include "Engine/World.h"

#include "Interactable.h"

void URaycastInteractableActivator::Scan_Implementation()
{
    FHitResult result{};
    FVector startLocation = GetComponentLocation();
    FVector endLocation = startLocation + (GetComponentRotation().Vector() * scanDistance);

    world->LineTraceSingleByChannel(
        result,
        startLocation,
        endLocation,
        collisionChannel
    );

    static AInteractable* _last = nullptr;
    static bool _activated = false;
    if(result.bBlockingHit)
    {
        if(_last != result.GetActor())
        {
            _activated = true;
            if(auto interactable = Cast<AInteractable>(result.GetActor()))
            {
                _last = interactable;
                if(interactableActivatedDelegate.IsBound())
                {
                    interactableActivatedDelegate.Execute(interactable);
                }
            }
        }

        //DrawDebugLine(GetWorld(), startLocation, endLocation, FColor::Green, false, 0.5f, 0, 1.0f);
    }
    else
    {
        if(_activated)
        {
            if(interactableDeactivatedDelegate.IsBound())
            {
                interactableDeactivatedDelegate.Execute(_last);
            }
            _activated = false;
            _last = nullptr;
        }

        //DrawDebugLine(GetWorld(), startLocation, endLocation, FColor::Red, false, 0.5f, 10, 1.0f);
    }
}